;; Guile bindings for GNU Gettext's libgettextpo.
;; Copyright © 2019 Miguel Ángel Arruga Vivas <rosen644835@gmail.com>
;;
;; This is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 3 of the License, or
;; (at your option) any later version.
;;
;; This software is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this software.  If not, see <http://www.gnu.org/licenses/>.

;;; Commentary:
;;
;; Error management for libgettextpo.
;;

;;; Code:
(define-module (gettext po error)
  #:use-module (gettext po internal-api)
  #:use-module (gettext po message)
  #:use-module (srfi srfi-9)
  #:use-module (ice-9 format)
  #:use-module (system foreign)
  #:export (<po-error-location>
	    po-error-location?
	    make-po-error-location
	    po-error-location-filename
	    po-error-location-line
	    po-error-location-column

	    <po-error-message>
	    po-error-message?
	    make-po-error-message
	    po-error-message-severity
	    po-error-message-message-pointer
	    po-error-message-location
	    po-error-message-multiline?
	    po-error-message-text

	    make-throw-message-handler
	    call-with-xerror-handler))

(define-record-type <po-error-location>
  (make-po-error-location filename line column)
  po-error-location?
  (filename po-error-location-filename)
  (line po-error-location-line)
  (column po-error-location-column))

(define-record-type <po-error-message>
  (make-po-error-message severity message-pointer location multiline? text)
  po-error-message?
  (severity po-error-message-severity)
  (message-pointer po-error-message-message-pointer)
  (location po-error-message-location)
  (multiline? po-error-location-multiline?)
  (text po-error-message-text))

(define (int-severity->symbol severity)
  (cond ((eqv? severity PO_SEVERITY_WARNING) 'warning)
	((eqv? severity PO_SEVERITY_ERROR) 'warning)
	((eqv? severity PO_SEVERITY_FATAL_ERROR) 'fatal-error)
	(else severity)))

(define* (%print-message-handler msg #:optional msg2)
  (format #t "Message: ~a~%" msg)
  (when msg2
    (format #t "Message 2: ~a~%" msg2)))

(define %exception-type 'libgettextpo-error)

(define* (make-throw-message-handler #:optional type)
  (let ((exception-type (if type type %exception-type)))
    (lambda args
      (apply throw exception-type args))))

(define* (call-with-xerror-handler function args #:key (error-handler #f)
				   (exception-handler #f))
  (let ((error-handler (or error-handler %print-message-handler)))
    (define (make-xerror)
      (define (parse severity-int message-ptr fname-ptr lineno
		     column multiline-int message-text-ptr)
	(let ((message (and message-ptr
			    (not (null-pointer? message-ptr))
			    (make-po-message #:pointer message-ptr)))
	      (multiline? (not (eqv? multiline-int 0)))
	      (filename (and fname-ptr
			     (not (null-pointer? fname-ptr))
			     (pointer->string fname-ptr)))
	      (message-text (and message-text-ptr
				 (not (null-pointer? message-text-ptr))
				 (pointer->string message-text-ptr))))
	  (let ((severity (int-severity->symbol severity-int))
		(location (and filename
			       (make-po-error-location filename lineno column))))
	    (make-po-error-message severity message location
				   multiline? message-text))))
      (define (xerror1)
	(define (cb severity message-ptr fname-ptr lineno column
		    multiline-int message-text-ptr)
	  (let ((msg (parse severity message-ptr fname-ptr lineno
			    column multiline-int message-ptr)))
	    (when (eqv? severity PO_SEVERITY_FATAL_ERROR)
	      (throw %exception-type msg)) ; Non local exit
	    (error-handler msg)))
	(procedure->pointer void cb (list int '* '* size_t size_t int '*)))
      (define (xerror2)
	(define (cb severity message-ptr-1 fname-ptr-1 lineno-1 column-1
		    multiline-int-1 message-text-ptr-1
		    message-ptr-2 fname-ptr-2 lineno-2 column-2
		    multiline-int-2 message-text-ptr-2)
	  (let ((msg1 (parse severity message-ptr-1 fname-ptr-1 lineno-1 column-1
			     multiline-int-1 message-text-ptr-1))
		(msg2 (parse severity message-ptr-2 fname-ptr-2 lineno-2 column-2
			     multiline-int-2 message-text-ptr-2)))
	    (when (eqv? severity PO_SEVERITY_FATAL_ERROR)
	      (throw %exception-type msg1 msg2)) ; Non local exit
	    (error-handler msg1 msg2)))
	(procedure->pointer void cb (list int '* '* size_t size_t int '*
					  '* '* size_t size_t int '*)))
      (list (xerror1) (xerror2)))

    ;; TODO: Create the structure with guile and call directly the
    ;; function here instead of the C wrapper.
    (catch %exception-type
      (lambda ()
	(apply function (append args (make-xerror))))
      (let ((exception-handler (or exception-handler %print-message-handler)))
	(lambda (key . exception-args)
	  (apply exception-handler exception-args)
	  #f))))) ; Is this enough?

