;; Guile bindings for GNU Gettext's libgettextpo.
;; Copyright © 2019 Miguel Ángel Arruga Vivas <rosen644835@gmail.com>
;;
;; This is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 3 of the License, or
;; (at your option) any later version.
;;
;; This software is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this software.  If not, see <http://www.gnu.org/licenses/>.

;;; Commentary:
;;
;; Scheme API for po_message_iterator_t from libgettextpo.
;;

;;; Code:
(define-module (gettext po iterator)
  #:use-module (gettext po format)
  #:use-module (gettext po file)
  #:use-module (gettext po filepos)
  #:use-module (gettext po header)
  #:use-module (gettext po internal-api)
  #:use-module (gettext po message)
  #:use-module (srfi srfi-1)
  #:use-module (srfi srfi-9)
  #:use-module (system foreign)
  #:export (<po-message-iterator>
	    po-message-iterator?
	    po-message-iterator-po-file
	    po-message-iterator-domain
	    po-message-iterator-pointer
	    po-message-iterator-eqv?

	    make-po-message-iterator
	    po-next-message
	    po-message-insert!
	    po-message-append!
	    po-message-remove

	    fold-po-messages
	    po-file-messages
	    filter-po-file
	    for-each-message))

(define-record-type <po-message-iterator>
  (make-po-message-iterator-object po-file domain pointer)
  po-message-iterator?
  (po-file po-message-iterator-po-file)
  (domain po-message-iterator-domain)
  (pointer po-message-iterator-pointer))

(define (po-message-iterator-eqv? lhs rhs)
  "Pointer comparison for <po-message-iterator> objects."
  (let ((lhs-pointer (po-message-iterator-pointer lhs))
	(rhs-pointer (po-message-iterator-pointer rhs)))
    (eqv? (pointer-address lhs-pointer) (pointer-address rhs-pointer))))

;; Po message iterator
(define (wrap-po-message-iterator-pointer ptr)
  (and ptr (not (null-pointer? ptr))
       (make-pointer (pointer-address ptr) %po-message-iterator-free)))

(define* (make-po-message-iterator po-file #:key (domain #f))
  (let* ((pointer (po-file-pointer po-file))
	 (real-domain (if domain domain %null-pointer))
	 (result (wrap-po-message-iterator-pointer
		  (%po-message-iterator pointer real-domain))))
    (and result (make-po-message-iterator-object po-file domain result))))

(define (po-next-message iterator)
  (let* ((pointer (po-message-iterator-pointer iterator))
	 (result (%po-next-message pointer)))
    (make-po-message #:from-iterator iterator #:pointer result)))

;; Scheme-only API
(define* (fold-po-messages kons init po-file #:key (domain #f))
  (let ((iterator (make-po-message-iterator po-file #:domain domain)))
    (let loop ((last init)
	       (message (po-next-message iterator)))
      (if message
	  (loop (kons message last) (po-next-message iterator))
	  last))))

(define* (po-file-messages po-file #:key (domain #f))
  (reverse (fold-po-messages cons '() po-file #:domain domain)))

(define (copy-po-message message)
  (let ((new-message (make-po-message)))
    ;; Copy normal fields
    (let ((msgctxt (po-message-msgctxt message))
	  (msgid (po-message-msgid message))
	  (msgid-plural (po-message-msgid-plural message))
	  (msgstr (po-message-msgstr message))
	  (comments (po-message-comments message))
	  (extracted-comments (po-message-extracted-comments message))
	  (prev-msgctxt (po-message-prev-msgctxt message))
	  (prev-msgid (po-message-prev-msgid message))
	  (prev-msgid-plural (po-message-prev-msgid-plural message))
	  (is-obsolete? (po-message-is-obsolete? message))
	  (is-fuzzy? (po-message-is-fuzzy? message))
	  (is-range? (po-message-is-range? message)))
      (po-message-set-obsolete! new-message is-obsolete?)
      (po-message-set-fuzzy! new-message is-fuzzy?)
      (when msgctxt
	(po-message-set-msgctxt! new-message msgctxt))
      (when msgid
	(po-message-set-msgid! new-message msgid))
      (when msgid-plural
	(po-message-set-msgid-plural! new-message msgid-plural))
      (when msgstr
	(po-message-set-msgstr! new-message msgstr))
      (when comments
	(po-message-set-comments! new-message comments))
      (when extracted-comments
	(po-message-set-extracted-comments! new-message extracted-comments))
      (when prev-msgctxt
	(po-message-set-prev-msgctxt! new-message prev-msgctxt))
      (when prev-msgid
	(po-message-set-prev-msgid! new-message prev-msgid))
      (when prev-msgid-plural
	(po-message-set-prev-msgid-plural! new-message prev-msgid-plural))
      (when is-range?
	(po-message-set-range! new-message (car is-range?) (cdr is-range?))))

    ;; Copy plurals
    (let* ((from-iterator (po-message-from-iterator message))
	   (domain (po-message-iterator-domain from-iterator))
	   (po-file (po-message-iterator-po-file from-iterator))
	   (nplurals (po-file-nplurals po-file #:domain domain)))
      (for-each (lambda (i)
		  (let ((msgstr-plural (po-message-msgstr-plural message i)))
		    (when msgstr-plural
		      (po-message-set-msgstr-plural! new-message i
						     msgstr-plural))))
		(iota nplurals)))

    ;; Copy formats
    (for-each (lambda (format)
		(let ((format? (po-message-is-format? message format)))
		  (po-message-set-format! new-message format format?)))
	      (po-format-list))

    ;; Copy filepos
    (for-each (lambda (filepos)
		(let ((file (po-filepos-file filepos))
		      (line (po-filepos-start-line filepos)))
		  (po-message-add-filepos! new-message file line)))
	      (po-message-all-filepos message))

    ;; Return created message
    new-message))

(define (po-message-insert! iterator message)
  ;; Do not insert the same message twice
  (let ((message (if (po-message-from-iterator message)
		     (copy-po-message message)
		     message)))
    (let ((message-pointer (po-message-pointer message))
	  (iterator-pointer (po-message-iterator-pointer iterator)))
      (%po-message-insert iterator-pointer message-pointer)
      (po-message-set-from-iterator! message iterator)
      message)))

(define (po-message-append! po-file message)
  (define (last-iterator)
    (let ((iterator (make-po-message-iterator po-file)))
      (let loop ((message (po-next-message iterator)))
	(if message
	    (loop (po-next-message iterator))
	    iterator))))
  (po-message-insert! (last-iterator) message))

(define* (filter-po-file filter po-file)
  (define (insert message iterator)
    (when (filter message)
      (po-message-insert! iterator message)))
  (define (make-domain-inserter file)
    (lambda (domain)
      (let ((iterator (make-po-message-iterator file #:domain domain)))
	(fold-po-messages insert iterator po-file #:domain domain))))
  (let ((domains (po-file-domains po-file))
	(new-file (make-po-file)))
    (for-each (make-domain-inserter new-file) domains)
    new-file))

;; XXX: This interface probably isn't there for a reason... Maybe it's
;; not a good idea to export it.  At least the documentation and the
;; missing exclamation mark can be hints of the resources involved.
(define* (po-message-remove po-file message #:key (domain #f))
  "Generate a new file without MESSAGE in domain."
  (define (this-message? msg)
    (po-message-eqv? msg message))
  (and (find this-message? (po-file-messages po-file #:domain domain))
       (filter-po-file (negate this-message?) po-file)))
