;; Guile bindings for GNU Gettext's libgettextpo.
;; Copyright © 2019 Miguel Ángel Arruga Vivas <rosen644835@gmail.com>
;;
;; This is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 3 of the License, or
;; (at your option) any later version.
;;
;; This software is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this software.  If not, see <http://www.gnu.org/licenses/>.

;;; Commentary:
;;
;; Scheme API for po_message_t from libgettextpo.
;;

;;; Code:
(define-module (gettext po message)
  #:use-module (gettext po internal-api)
  #:use-module (ice-9 match)
  #:use-module (srfi srfi-9)
  #:use-module (srfi srfi-11)
  #:use-module (system foreign)
  #:export (<po-message>
	    po-message?
	    po-message-from-iterator
	    po-message-set-from-iterator!
	    po-message-pointer
	    po-message-eqv?

	    ;; Po message API
	    make-po-message
	    po-message-msgctxt
	    po-message-set-msgctxt!
	    po-message-msgid
	    po-message-set-msgid!
	    po-message-msgid-plural
	    po-message-set-msgid-plural!
	    po-message-msgstr
	    po-message-set-msgstr!
	    po-message-msgstr-plural
	    po-message-set-msgstr-plural!
	    po-message-comments
	    po-message-set-comments!
	    po-message-extracted-comments
	    po-message-set-extracted-comments!
	    po-message-prev-msgctxt
	    po-message-set-prev-msgctxt!
	    po-message-prev-msgid
	    po-message-set-prev-msgid!
	    po-message-prev-msgid-plural
	    po-message-set-prev-msgid-plural!
	    po-message-is-obsolete?
	    po-message-set-obsolete!
	    po-message-is-fuzzy?
	    po-message-set-fuzzy!
	    po-message-is-format?
	    po-message-set-format!
	    po-message-is-range?
	    po-message-set-range!

	    ;; Helper functions
	    po-message-translated?))

(define-record-type <po-message>
  (make-po-message-object from-iterator pointer)
  po-message?
  (from-iterator po-message-from-iterator po-message-set-from-iterator!)
  (pointer po-message-pointer))

(define (po-message-eqv? lhs rhs)
  "Pointer comparison for <po-message> objects."
  (let ((lhs-pointer (po-message-pointer lhs))
	(rhs-pointer (po-message-pointer rhs)))
    (eqv? (pointer-address lhs-pointer) (pointer-address rhs-pointer))))

(define* (make-po-message #:key (from-iterator #f) (pointer #f))
  "Create a new message.  It must be added to a <po-file> with
po-message-insert! in order to not leak the memory."
  (cond (pointer
	 (and (not (null-pointer? pointer))
	      (make-po-message-object from-iterator pointer)))
	(from-iterator
	 (throw 'make-po-message-invalid-args
		"A message from an iterator must have a pointer"))
	(else
	 (let ((pointer (%po-message-create)))
	   (and (not (null-pointer? pointer))
		(make-po-message-object #f pointer))))))

(define (po-message-msgctxt message)
  (let* ((pointer (po-message-pointer message))
	 (cvalue (%po-message-msgctxt pointer)))
    (and (not (null-pointer? cvalue))
	 (pointer->string cvalue))))
(define (po-message-set-msgctxt! message value)
  (let ((pointer (po-message-pointer message))
	(cvalue (string->pointer value)))
    (%po-message-set-msgctxt! pointer cvalue)
    #t))

(define (po-message-msgid message)
  (let* ((pointer (po-message-pointer message))
	 (cvalue (%po-message-msgid pointer)))
    (and (not (null-pointer? cvalue))
	 (pointer->string cvalue))))
(define (po-message-set-msgid! message value)
  (let ((pointer (po-message-pointer message))
	(cvalue (string->pointer value)))
    (%po-message-set-msgid! pointer cvalue)
    #t))

(define (po-message-msgid-plural message)
  (let* ((pointer (po-message-pointer message))
	 (cvalue (%po-message-msgid-plural pointer)))
    (and (not (null-pointer? cvalue))
	 (pointer->string cvalue))))
(define (po-message-set-msgid-plural! message value)
  (let ((pointer (po-message-pointer message))
	(cvalue (string->pointer value)))
    (%po-message-set-msgid-plural! pointer cvalue)
    #t))

(define (po-message-msgstr message)
  (let* ((pointer (po-message-pointer message))
	 (cvalue (%po-message-msgstr pointer)))
    (and (not (null-pointer? cvalue))
	 (pointer->string cvalue))))
(define (po-message-set-msgstr! message value)
  (let ((pointer (po-message-pointer message))
	(cvalue (string->pointer value)))
    (%po-message-set-msgstr! pointer cvalue)
    #t))

(define (po-message-msgstr-plural message index)
  (let* ((pointer (po-message-pointer message))
	 (cvalue (%po-message-msgstr-plural pointer index)))
    (and (not (null-pointer? cvalue))
	 (pointer->string cvalue))))
(define (po-message-set-msgstr-plural! message index value)
  (let ((pointer (po-message-pointer message))
	(cvalue (string->pointer value)))
    (%po-message-set-msgstr-plural! pointer index cvalue)
    #t))

(define (po-message-comments message)
  (let* ((pointer (po-message-pointer message))
	 (cvalue (%po-message-comments pointer)))
    (and (not (null-pointer? cvalue))
	 (pointer->string cvalue))))
(define (po-message-set-comments! message value)
  (let ((pointer (po-message-pointer message))
	(cvalue (string->pointer value)))
    (%po-message-set-comments! pointer cvalue)
    #t))

(define (po-message-extracted-comments message)
  (let* ((pointer (po-message-pointer message))
	 (cvalue (%po-message-extracted-comments pointer)))
    (and (not (null-pointer? cvalue))
	 (pointer->string cvalue))))
(define (po-message-set-extracted-comments! message value)
  (let ((pointer (po-message-pointer message))
	(cvalue (string->pointer value)))
    (%po-message-set-extracted-comments! pointer cvalue)
    #t))

(define (po-message-prev-msgctxt message)
  (let* ((pointer (po-message-pointer message))
	 (cvalue (%po-message-prev-msgctxt pointer)))
    (and (not (null-pointer? cvalue))
	 (pointer->string cvalue))))
(define (po-message-set-prev-msgctxt! message value)
  (let ((pointer (po-message-pointer message))
	(cvalue (string->pointer value)))
    (%po-message-set-prev-msgctxt! pointer cvalue)
    #t))

(define (po-message-prev-msgid message)
  (let* ((pointer (po-message-pointer message))
	 (cvalue (%po-message-prev-msgid pointer)))
    (and (not (null-pointer? cvalue))
	 (pointer->string cvalue))))
(define (po-message-set-prev-msgid! message value)
  (let ((pointer (po-message-pointer message))
	(cvalue (string->pointer value)))
    (%po-message-set-prev-msgid! pointer cvalue)
    #t))

(define (po-message-prev-msgid-plural message)
  (let* ((pointer (po-message-pointer message))
	 (cvalue (%po-message-prev-msgid-plural pointer)))
    (and (not (null-pointer? cvalue))
	 (pointer->string cvalue))))
(define (po-message-set-prev-msgid-plural! message value)
  (let ((pointer (po-message-pointer message))
	(cvalue (string->pointer value)))
    (%po-message-set-prev-msgid-plural! pointer cvalue)
    #t))

(define (c-int->boolean val) (not (eqv? 0 val)))
(define (boolean->c-int val) (if val 1 0))

(define (po-message-is-obsolete? message)
  (let* ((pointer (po-message-pointer message))
	 (cvalue (%po-message-is-obsolete? pointer)))
    (c-int->boolean cvalue)))
(define (po-message-set-obsolete! message value)
  (let ((pointer (po-message-pointer message))
	(cvalue (boolean->c-int value)))
    (%po-message-set-obsolete! pointer cvalue)
    #t))

(define (po-message-is-fuzzy? message)
  (let* ((pointer (po-message-pointer message))
	 (cvalue (%po-message-is-fuzzy? pointer)))
    (c-int->boolean cvalue)))
(define (po-message-set-fuzzy! message value)
  (let ((pointer (po-message-pointer message))
	(cvalue (boolean->c-int value)))
    (%po-message-set-fuzzy! pointer cvalue)
    #t))

(define (po-message-is-format? message format)
  (let* ((pointer (po-message-pointer message))
	 (format-pointer (string->pointer format))
	 (cvalue (%po-message-is-format? pointer format-pointer)))
    (c-int->boolean cvalue)))
(define (po-message-set-format! message format value)
  (let ((pointer (po-message-pointer message))
	(format-pointer (string->pointer format))
	(cvalue (boolean->c-int value)))
    (%po-message-set-format! pointer format-pointer cvalue)
    #t))

(define (po-message-is-range? message)
  (let ((pointer (po-message-pointer message)))
    (%po-message-is-range? pointer)))

(define (po-message-set-range! message min max)
  (let ((pointer (po-message-pointer message)))
    (%po-message-set-range! pointer min max)))


;; Helper function
(define (po-message-translated? message)
  (let ((msgstr (po-message-msgstr message)))
    (not (or (not msgstr) (string=? "" msgstr)))))

