/* Guile bindings for GNU Gettext's libgettextpo.
   Copyright © 2019 Miguel Ángel Arruga Vivas <rosen644835@gmail.com>

   This is free software; you can redistribute it and/or modify it
   under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 3 of the License, or
   (at your option) any later version.

   This software is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this software.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include <libguile.h>
#include <gettext-po.h>

SCM_VARIABLE_INIT (scm_po_severity_warning, "PO_SEVERITY_WARNING",
		   scm_from_int (PO_SEVERITY_WARNING));
SCM_VARIABLE_INIT (scm_po_severity_error, "PO_SEVERITY_ERROR",
		   scm_from_int (PO_SEVERITY_ERROR));
SCM_VARIABLE_INIT (scm_po_severity_fatal_error, "PO_SEVERITY_FATAL_ERROR",
		   scm_from_int (PO_SEVERITY_FATAL_ERROR));

SCM_DEFINE (scm_gettextpo_po_file_read, "%po-file-read",
	    3, 0, 0, (SCM filename, SCM xerror_handler, SCM xerror2_handler),
	    "Read FILENAME and return the wrapped pointer.  "
	    "xerror_handler and xerror2_handler must not exit locally when "
            "severity flag is PO_SEVERITY_FATAL_ERROR.")
{
  char* filename_ptr;
  po_file_t po_file;
  SCM ret, pointer;
  scm_t_pointer_finalizer deleter;
  struct po_xerror_handler handler =
    {
     .xerror = scm_to_pointer (xerror_handler),
     .xerror2 = scm_to_pointer (xerror2_handler)
    };

  scm_dynwind_begin (SCM_F_DYNWIND_REWINDABLE);

  filename_ptr = scm_to_locale_string (filename);
  scm_dynwind_free (filename_ptr);

  po_file = po_file_read (filename_ptr, &handler);

  if (po_file)
    deleter = (scm_t_pointer_finalizer) po_file_free;
  else
    deleter = NULL;

  ret = scm_from_pointer (po_file, deleter);
  scm_dynwind_end ();

  return ret;
}

SCM_DEFINE (scm_gettextpo_po_file_write, "%po-file-write",
	    4, 0, 0, (SCM po_file, SCM filename, SCM xerror_handler,
		      SCM xerror2_handler),
	    "Write PO_FILE into FILENAME and return the boolean value.  "
	    "xerror_handler and xerror2_handler must not exit "
            "locally when severity flag is PO_SEVERITY_FATAL_ERROR.")
{
  char* filename_ptr;
  po_file_t file_ptr, result;
  int stored_errno;
  SCM ret;
  struct po_xerror_handler handler =
    {
     .xerror = scm_to_pointer (xerror_handler),
     .xerror2 = scm_to_pointer (xerror2_handler)
    };

  scm_dynwind_begin (SCM_F_DYNWIND_REWINDABLE);

  file_ptr = scm_to_pointer (po_file);
  filename_ptr = scm_to_locale_string (filename);
  scm_dynwind_free (filename_ptr);

  result = po_file_write (file_ptr, filename_ptr, &handler);

  if (result == file_ptr)
    ret = SCM_BOOL_T;
  else
    ret = SCM_BOOL_F;
  scm_dynwind_end ();

  return ret;
}

SCM_DEFINE (scm_gettextpo_po_file_domains, "%po-file-domains",
	    1, 0, 0, (SCM po_file),
	    "Retrieve the domains from the po file, in reverse order.")
{
  SCM ret;
  po_file_t file;
  const char* const* domains;
  const char* domain;

  file = (po_file_t) scm_to_pointer (po_file);
  if (!file)
    return SCM_BOOL_F;

  domains = po_file_domains (file);
  if (!domains)
    return SCM_BOOL_F;

  ret = SCM_EOL;
  for (domain = *domains; domain; domain = *(++domains))
    ret = scm_cons (scm_from_locale_string (domain), ret);
  return ret;
}

SCM_DEFINE (scm_gettextpo_po_header_field, "%po-header-field",
	    2, 0, 0, (SCM header, SCM field),
	    "Return the header field as a string..")
{
  SCM ret;
  po_file_t file;
  char *header_ptr;
  char *field_ptr;
  char *value_ptr;

  scm_dynwind_begin (0);

  header_ptr = scm_to_locale_string (header);
  scm_dynwind_free (header_ptr);

  field_ptr = scm_to_locale_string (field);
  scm_dynwind_free (field_ptr);

  value_ptr = po_header_field (header_ptr, field_ptr);
  if (value_ptr)
    ret = scm_take_locale_string (value_ptr);
  else
    ret = SCM_BOOL_F;

  scm_dynwind_end ();
  return ret;
}

SCM_DEFINE (scm_gettextpo_po_header_set_field, "%po-header-set-field",
	    3, 0, 0, (SCM header, SCM field, SCM value),
	    "Return the header field as a string..")
{
  SCM ret;
  char *header_ptr;
  char *field_ptr;
  char *value_ptr;
  char *new_header;

  scm_dynwind_begin (0);

  header_ptr = scm_to_locale_string (header);
  scm_dynwind_free (header_ptr);

  field_ptr = scm_to_locale_string (field);
  scm_dynwind_free (field_ptr);

  value_ptr = scm_to_locale_string (value);
  scm_dynwind_free (value_ptr);

  new_header = po_header_set_field (header_ptr, field_ptr, value_ptr);
  if (new_header)
    ret = scm_take_locale_string (new_header);
  else
    ret = SCM_BOOL_F;

  scm_dynwind_end ();
  return ret;
}

SCM_DEFINE (scm_gettextpo_po_message_is_range, "%po-message-is-range?",
	    1, 0, 0, (SCM po_message),
	    "Check the numeric range of the message and return it as "
	    "a pair of (min . max), @code{#f} otherwise.")
{
  int min, max;
  po_message_t message;

  message = (po_message_t) scm_to_pointer (po_message);
  if (!message)
    return SCM_BOOL_F;

  if (po_message_is_range (message, &min, &max))
    return scm_cons (scm_from_int (min), scm_from_int (max));
  return SCM_BOOL_F;
}

SCM_DEFINE (scm_gettextpo_po_format_list, "%po-format-list",
	    0, 0, 0, (), "List of available formats.")
{
  SCM ret;
  const char* const* formats;
  const char* format;

  formats = po_format_list ();
  if (!formats)
    return SCM_BOOL_F;

  ret = SCM_EOL;
  for (format = *formats; format; format = *(++formats))
    ret = scm_cons (scm_from_locale_string (format), ret);
  return ret;
}

SCM_DEFINE (scm_gettextpo_po_file_check_all, "%po-file-check-all",
	    3, 0, 0, (SCM po_file, SCM xerror_handler, SCM xerror2_handler),
	    "Check an entire PO file.")
{
  po_file_t file_ptr;
  struct po_xerror_handler handler =
    {
     .xerror = scm_to_pointer (xerror_handler),
     .xerror2 = scm_to_pointer (xerror2_handler)
    };

  file_ptr = (po_file_t) scm_to_pointer (po_file);
  if (!file_ptr)
    return SCM_BOOL_F;

  po_file_check_all (file_ptr, &handler);
  return SCM_BOOL_T;
}

SCM_DEFINE (scm_gettextpo_po_message_check_all, "%po-message-check-all",
	    4, 0, 0,
	    (SCM message, SCM iterator, SCM xerror_handler, SCM xerror2_handler),
	    "Check a single message to be inserted in a PO file.")
{
  po_message_t message_ptr;
  po_message_iterator_t iterator_ptr;
  struct po_xerror_handler handler =
    {
     .xerror = scm_to_pointer (xerror_handler),
     .xerror2 = scm_to_pointer (xerror2_handler)
    };

  message_ptr = (po_message_t) scm_to_pointer (message);
  iterator_ptr = (po_message_iterator_t) scm_to_pointer (iterator);

  if (!message_ptr || !iterator_ptr)
    return SCM_BOOL_F;

  po_message_check_all (message_ptr, iterator_ptr, &handler);
  return SCM_BOOL_T;
}

SCM_DEFINE (scm_gettextpo_po_message_check_format, "%po-message-check-format",
	    3, 0, 0, (SCM message, SCM xerror_handler, SCM xerror2_handler),
	    "Check the message format string.")
{
  po_message_t message_ptr;
  struct po_xerror_handler handler;

  handler.xerror = scm_to_pointer (xerror_handler);
  handler.xerror2 = scm_to_pointer (xerror2_handler);
  message_ptr = (po_message_t) scm_to_pointer (message);

  if (!message_ptr)
    return SCM_BOOL_F;

  po_message_check_format (message_ptr, &handler);
  return SCM_BOOL_T;
}

void
init_gettextpo_extension ()
{
#include "extension.x"
}

