;; Guile bindings for GNU Gettext's libgettextpo.
;; Copyright © 2019 Miguel Ángel Arruga Vivas <rosen644835@gmail.com>
;;
;; This is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 3 of the License, or
;; (at your option) any later version.
;;
;; This software is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this software.  If not, see <http://www.gnu.org/licenses/>.

;;; Commentary:
;;
;; The examples from the manual must work to be useful, and for that
;; they must be tested.
;;

;;; Code:
(define-module (test-manual)
  #:use-module (gettext po)
  #:use-module (ice-9 regex)
  #:use-module (ice-9 format)
  #:use-module (srfi srfi-1)
  #:use-module (srfi srfi-2)
  #:use-module (srfi srfi-19)
  #:use-module (srfi srfi-64))

(define %abs-top-srcdir (getenv "abs_top_srcdir"))
(define %tests-data-dir (getenv "tests_data_dir"))
(define %po-file-defuzzy
  (string-append %abs-top-srcdir "/" %tests-data-dir "/fuzzy.po"))
(define %po-file-template
  (string-append %abs-top-srcdir "/" %tests-data-dir "/template.po"))
(define %po-file-temp (string-append %tests-data-dir "/t-manual.po"))

(test-begin "manual")

(test-group-with-cleanup "manual-remove-fuzzy"
  (define execution-success #f)
  (copy-file %po-file-defuzzy %po-file-temp)
  ;; The file cannot be written in distcheck
  (chmod %po-file-temp #o666)
  (test-group "manual-remove-fuzzy-check-input"
    (let ((po (read-po-file %po-file-temp)))
      (test-assert "manual-remove-fuzzy-check-input-readable" po)
      (let ((messages (po-file-messages po)))
	(test-eqv "manual-remove-fuzzy-check-input-length"
	  3 (length messages))
	(test-eqv "manual-remove-fuzzy-check-input-fuzzy"
	  2 (count po-message-is-fuzzy? messages))
	(test-eqv "manual-remove-fuzzy-check-input-not-fuzzy"
	  1 (count (negate po-message-is-fuzzy?) messages)))))

  (test-assert "manual-remove-fuzzy-execution"
    (let ((success #t))
      ;; Code from manual
      (with-po-files ((po-file %po-file-temp))
        (for-each (lambda (message)
		    ;; Remove the fuzzy flag from all messages
		    (po-message-set-fuzzy! message #f))
		  (po-file-messages po-file)))
      (set! execution-success success)
      ;; End code from manual
      success))

  (unless execution-success (test-skip 1))
  (test-group "manual-remove-fuzzy-result"
    (let ((po (read-po-file %po-file-temp)))
      (test-assert "manual-remove-fuzzy-write" po)
      (let ((messages (po-file-messages po)))
	(test-eqv "manual-remove-fuzzy-messages-length"
	  3 (length messages))
	(test-eqv "manual-remove-fuzzy-messages-fuzzy"
	  0 (count po-message-is-fuzzy? messages))
	(test-eqv "manual-remove-fuzzy-messages-not-fuzzy"
	  3 (count (negate po-message-is-fuzzy?) messages)))))
  (delete-file %po-file-temp))

(test-group-with-cleanup "manual-template-translation"
  (define execution-success #f)
  (copy-file %po-file-template %po-file-temp)
  ;; The file cannot be written in distcheck
  (chmod %po-file-temp #o666)
  (test-group "manual-template-translation-check-input"
    (let ((po (read-po-file %po-file-temp)))
      (test-assert "manual-template-translation-check-input-readable" po)
      (let ((messages (po-file-messages po)))
	(test-eqv "manual-template-translation-check-input-length"
	  10 (length messages))
	(test-eqv "manual-template-translation-check-input-fuzzy"
	  1 (count po-message-is-fuzzy? messages))
	(test-eqv "manual-template-translation-check-input-obsolete"
	  1 (count po-message-is-obsolete? messages))
	(test-eqv "manual-template-translation-check-input-translated"
	  1 (count po-message-translated? messages)))))

  (test-assert "manual-template-translation-execution"
    (let ((success #t))
      ;; Code from manual
      (define translate-template
	(let ((rx (make-regexp "^Template ([^ ]*) translation.$"
			       regexp/extended))
	      (translated "~a translated template."))
	  (lambda (message)
	    (unless (po-message-translated? message)
	      (and-let* ((msgid (po-message-msgid message))
			 (result (regexp-exec rx msgid))
			 (data (match:substring result 1)))
		(po-message-set-msgstr! message (format #f translated data)))))))
      (with-po-files ((po-file %po-file-temp))
        (for-each translate-template (po-file-messages po-file)))
     ;; End code from manual
     (set! execution-success success)
     success))

  (unless execution-success (test-skip 1))
  (test-group "manual-template-translation-result"
    (let ((po (read-po-file %po-file-temp)))
      (test-assert "manual-template-translation-write" po)
      (let ((messages (po-file-messages po)))
	(test-eqv "manual-template-translation-check-input-length"
	  10 (length messages))
	(test-eqv "manual-template-translation-check-input-fuzzy"
	  1 (count po-message-is-fuzzy? messages))
	(test-eqv "manual-template-translation-check-input-obsolete"
	  1 (count po-message-is-obsolete? messages))
	(test-eqv "manual-template-translation-check-input-translated"
	  5 (count po-message-translated? messages)))))
  (delete-file %po-file-temp))

(test-end "manual")
