;; Guile bindings for GNU Gettext's libgettextpo.
;; Copyright © 2019 Miguel Ángel Arruga Vivas <rosen644835@gmail.com>
;;
;; This is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 3 of the License, or
;; (at your option) any later version.
;;
;; This software is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;; General Public License for more details.
;;
;; You should have received a copy of the GNU General Public License
;; along with this software.  If not, see <http://www.gnu.org/licenses/>.

;;; Commentary:
;;
;; Basic PO file reading.
;;

;;; Code:
(define-module (test-read)
  #:use-module (gettext po)
  #:use-module (srfi srfi-1)
  #:use-module (srfi srfi-19)
  #:use-module (srfi srfi-64)
  #:use-module ((system foreign) #:select (null-pointer?)))

(test-begin "read")

(define %abs-top-srcdir (getenv "abs_top_srcdir"))
(define %tests-data-dir (getenv "tests_data_dir"))
(define %po-file-complete-filename
  (string-append %abs-top-srcdir "/" %tests-data-dir "/complete.po"))

(define po-file-complete #f)
(define po-file-complete-iterator #f)
(test-group "read-complete"
  (let ((complete (read-po-file %po-file-complete-filename)))
    (test-assert "read-complete file read" complete)
    (set! po-file-complete complete)))

(unless po-file-complete
  (test-skip 1))
(test-group "<po-file> data"
  (let ((pointer (po-file-pointer po-file-complete)))
    (test-assert "read-complete pointer" (not (null-pointer? pointer))))
  (let ((stored-fname (po-file-filename po-file-complete)))
    (test-equal "read-complete filename"
      %po-file-complete-filename stored-fname)))

(define po-file-complete-iterator #f)
(unless po-file-complete
  (test-skip 1))
(test-group "read-complete-iterator"
  (let ((iterator (make-po-message-iterator po-file-complete)))
    (test-assert "read-complete-iterator from file" iterator)
    (set! po-file-complete-iterator iterator)))

(unless po-file-complete-iterator
  (test-skip 1))
(test-group "<po-message-iterator> data"
  (let ((po-file (po-message-iterator-po-file po-file-complete-iterator)))
    (test-eq "read-complete-iterator po-file" po-file-complete po-file))
  (let ((domain (po-message-iterator-domain po-file-complete-iterator)))
    (test-assert "read-complete-iterator domain" (not domain)))
  (let ((pointer (po-message-iterator-pointer po-file-complete-iterator)))
    (test-assert "read-complete-iterator pointer-1" pointer)
    (test-assert "read-complete-iterator pointer-2"
      (not (null-pointer? pointer)))))

(define po-file-complete-header-message #f)
(unless po-file-complete-iterator
  (test-skip 1))
(test-group "read-complete-header"
  (let ((header (po-next-message po-file-complete-iterator)))
    (test-assert "read-complete-header-po-message?"
      (po-message? header))
    (test-equal "read-complete-header-msgid"
      "" (po-message-msgid header))
    (test-equal "read-complete-header-msgstr"
      (string-append "Project-Id-Version: Test 1.0\n"
		     "PO-Revision-Date: 1970-01-01 00:00+0000\n"
		     "Last-Translator: Translator <translator@example.or>\n"
		     "Language: ll\n"
		     "Language-Team: lang <lang@example.org>\n"
		     "MIME-Version: 1.0\n"
		     "Content-Type: text/plain; charset=ASCII\n"
		     "Content-Transfer-Encoding: 8bit\n"
		     "PluralForms: nplurals=3; plural=(n % 3)\n")
      (po-message-msgstr header))
    (test-equal "read-complete-header-comment"
      "Complete po file for tests\n"
      (po-message-comments header))
    (test-eqv "read-complete-header-nplurals"
      3 (po-file-nplurals po-file-complete))
    (set! po-file-complete-header-message header)))

(define po-file-complete-messages #f)
(unless (and po-file-complete-iterator po-file-complete-header-message)
  (test-skip 1))
(test-group "read-complete-all-messages"
  ;; po-file-messages is tested in iteration.scm, whereas
  ;; po-next-message is a C function, so avoid calling it here, even
  ;; though this is its basic implementation.
  (let ((messages (let loop ((last (list po-file-complete-header-message))
			     (next (po-next-message po-file-complete-iterator)))
		    (if next
			(loop (cons next last)
			      (po-next-message po-file-complete-iterator))
			(reverse last)))))
    (test-assert "read-complete-all-messages messages?" messages)
    (unless messages
      (test-skip 1))
    (test-group "read-complete-all-messages-tests"
      (test-eqv "read-complete-all-messages length" 11 (length messages))
      (test-assert "read-complete-all-messages po-message?"
	(fold (lambda (next last)
		(and last (po-message? next)))
	      #t messages))
      (test-assert "read-complete-all-messages from-iterator"
	(fold (lambda (next last)
		(and last (eq? po-file-complete-iterator
			       (po-message-from-iterator next))))
	      #t messages))
      (test-assert "read-complete-all-messages pointer"
	(fold (lambda (next last)
		(and last (not (null-pointer? (po-message-pointer next)))))
	      #t messages))
      (set! po-file-complete-messages messages))))

(define po-file-complete-msgid-list #f)
(unless po-file-complete-messages
  (test-skip 1))
(test-group "read-complete-all-msgids"
  (let ((msgid-list (map po-message-msgid po-file-complete-messages)))
    (test-assert "read-complete-msgid" msgid-list)
    (test-assert "read-complete-msgid member header"
      (member "" msgid-list))
    (test-assert "read-complete-msgid-member-1"
      (member "Test id" msgid-list))
    (test-assert "read-complete-msgid-member-2"
      (member "Test id singular" msgid-list))
    (test-assert "read-complete-msgid-member-3"
      (member "Test id singular with context" msgid-list))
    (test-assert "read-complete-msgid-member-4"
      (member "Test id fuzzy" msgid-list))
    (test-assert "read-complete-msgid-member-5"
      (member "Test id untranslated" msgid-list))
    (test-assert "read-complete-msgid-member-6"
      (member "Test id singular %d untranslated" msgid-list))
    (test-assert "read-complete-msgid-not-member-f"
      (not (member #f msgid-list)))
    (set! po-file-complete-msgid-list msgid-list)))

(define po-file-complete-msgid-plural-list #f)
(unless po-file-complete-messages
  (test-skip 1))
(test-group "read-complete-all-msgid-plural"
  (let ((msgid-plural-list (map po-message-msgid-plural
				po-file-complete-messages)))
    (test-assert "read-complete-msgid-plural" msgid-plural-list)
    (test-assert "read-complete-msgid-plural-member-1"
      (member "Test id plural" msgid-plural-list))
    (test-assert "read-complete-msgid-plural-member-2"
      (member "Test id plural with context" msgid-plural-list))
    (test-assert "read-complete-msgid-plural-member-3"
      (member "Test id plural untranslated" msgid-plural-list))
    (test-assert "read-complete-msgid-plural-member-4"
      (member "Test id plural %d untranslated" msgid-plural-list))
    (test-assert "read-complete-msgid-plural-member-5"
      (member #f msgid-plural-list))
    (set! po-file-complete-msgid-plural-list msgid-plural-list)))

(define po-file-complete-msgstr-list #f)
(unless po-file-complete-messages
  (test-skip 1))
(test-group "read-complete-all-msgstr"
  (let ((msgstr-list (map po-message-msgstr po-file-complete-messages)))
    (test-assert "read-complete-msgstr" msgstr-list)
    (test-assert "read-complete-msgstr-member-1"
      (member "Translated test id" msgstr-list))
    (test-assert "read-complete-msgstr-member-2"
      (member "Translated test id with ctxt" msgstr-list))
    (test-assert "read-complete-msgstr-member-3"
      (member "Translated test id fuzzy" msgstr-list))
    (set! po-file-complete-msgstr-list msgstr-list)))

(define po-file-complete-msgstr-plural-list #f)
(unless po-file-complete-messages
  (test-skip 1))
(test-group "read-complete-msgstr-plural"
  (define (plural-reader num)
    (lambda (x)
      (po-message-msgstr-plural x num)))
  (let ((plural-list '()))
    (test-group "read-complete-msgstr-plural 0"
      (let ((msgstr-plural-list-0 (map (plural-reader 0)
				       po-file-complete-messages)))
	(test-assert "read-complete-msgstr-plural-0" msgstr-plural-list-0)
	(test-assert "read-complete-msgstr-plural-0-member-1"
	  (member "Translated test id 0" msgstr-plural-list-0))
	(test-assert "read-complete-msgstr-plural-0-member-2"
	  (member "Translated test id with context 0" msgstr-plural-list-0))
	(test-assert "read-complete-msgstr-plural-0-member-3"
	  (member "Translated test id 0 %d" msgstr-plural-list-0))
	(set! plural-list (cons msgstr-plural-list-0 plural-list))))

    (test-group "read-complete-msgstr-plural 1"
      (let ((msgstr-plural-list-1 (map (plural-reader 1)
				       po-file-complete-messages)))
	(test-assert "read-complete-msgstr-plural-1" msgstr-plural-list-1)
	(test-assert "read-complete-msgstr-plural-1-member-1"
	  (member "Translated test id 1" msgstr-plural-list-1))
	(test-assert "read-complete-msgstr-plural-1-member-2"
	  (member "Translated test id with context 1" msgstr-plural-list-1))
	(test-assert "read-complete-msgstr-plural-1-member-3"
	  (member "Translated test id 1 %d" msgstr-plural-list-1))
	(set! plural-list (cons msgstr-plural-list-1 plural-list))))

    (test-group "read-complete-msgstr-plural 2"
      (let ((msgstr-plural-list-2 (map (plural-reader 2)
				       po-file-complete-messages)))
	(test-assert "read-complete-msgstr-plural-2" msgstr-plural-list-2)
	(test-assert "read-complete-msgstr-plural-2-member-1"
	  (member "Translated test id 2" msgstr-plural-list-2))
	(test-assert "read-complete-msgstr-plural-2-member-2"
	  (member "Translated test id with context 2" msgstr-plural-list-2))
	(set! plural-list (cons msgstr-plural-list-2 plural-list))))

    (unless (equal? plural-list '())
      (set! po-file-complete-msgstr-plural-list plural-list))))

(define po-file-complete-comments-list #f)
(unless po-file-complete-messages
  (test-skip 1))
(test-group "read-complete-all-comments"
  (let ((comments-list (map po-message-comments po-file-complete-messages)))
    (test-assert "read-complete-comments" comments-list)
    (test-assert "read-complete-comments-member-1"
      (member "Comment test id\n" comments-list))
    (test-assert "read-complete-comments-member-2"
      (member "Comment test id ngettext\n" comments-list))
    (test-assert "read-complete-comments-member-3"
      (member "Comment test id with context ngettext\n" comments-list))
    (test-assert "read-complete-comments-member-4"
      (member "Untranslated comment\n" comments-list))
    (test-assert "read-complete-comments-member-6"
      (member "Partially untranslated 1\n" comments-list))
    (test-assert "read-complete-comments-member-7"
      (member "Complete po file for tests\n" comments-list))
    (set! po-file-complete-comments-list comments-list)))

(define po-file-complete-extracted-comments-list #f)
(unless po-file-complete-messages
  (test-skip 1))
(test-group "read-complete-all-extracted-comments"
  (let ((extracted-comments-list (map po-message-extracted-comments
				      po-file-complete-messages)))
    (test-assert "read-complete-extracted-comments" extracted-comments-list)
    (test-assert "read-complete-extracted-comments-member-1"
      (member "Untranslated comment with plural\n" extracted-comments-list))
    (test-assert "read-complete-extracted-comments-member-2"
      (member "Partially untranslated 2\n" extracted-comments-list))
    (set! po-file-complete-extracted-comments-list extracted-comments-list)))

(test-assert "read-all-ok" (and po-file-complete
				po-file-complete-iterator
				po-file-complete-header-message
				po-file-complete-messages
				po-file-complete-msgid-plural-list
				po-file-complete-msgstr-list
				po-file-complete-msgstr-plural-list
				po-file-complete-comments-list
				po-file-complete-extracted-comments-list))

(test-end "read")
